<?php
/**
 * @package		Media Cache Fixer Joomla! 1.6/1.7/2.5/3.x
 * @type		Plugin (System)
 * @filename	mediacachefixer.php
 * @folder		<root>/plugins/system/mediacachefixer
 * @version     1.4
 * @modified	27-01-2020
 * @author		Web-expert.gr / Stergios Zgouletas
 * @website		http://www.web-expert.gr
 * @license		GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * 
 * @copyright (C) 2016 Web-expert.gr
 *
 * This program can be used under the terms of the GNU General Public License
 * as published by the Free Software Foundation, either version 3 of the License.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>
**/

defined('_JEXEC') or die('Restricted access');

jimport( 'joomla.plugin.plugin' );
class plgSystemMediacachefixer extends JPlugin
{

	function onAfterRender()
	{
		$app = JFactory::getApplication();
		$workOn=$this->params->get('workon',0);
		
		if(!$workOn || ($workOn==1 && $app->isAdmin()) ||  ($workOn==2 && !$app->isAdmin())) return;
		$doc=JFactory::getDocument();
		if(!method_exists($doc,'getHeadData')) return;
		
		$profiler = new JProfiler();				
		jimport('joomla.filesystem.folder');
		jimport('joomla.filesystem.file');
		
		$allowDebug=$this->params->get('debug',0);

		$session = JFactory::getSession();
		$checkEvery=(float)$this->params->get('checkevery',0)*60;
		$mode=(int)$this->params->get('mode',0);
		$buffer = JResponse::getBody();
		$headData = $doc->getHeadData();

		$headDataCheck=array('styleSheets'=>$headData['styleSheets'],'scripts'=>$headData['scripts'],'links'=>$headData['links']);
		
		$domain=rtrim(JURI::root(),'/');
		$debug=array();
		$now=time();
		$debug[]='Current DateTime:'.date('Y-m-d H:i',$now);
		foreach($headDataCheck as $key=>$MediaData)
		{
			foreach($MediaData as $oldUrl=>$mediaParams)
			{
				$newUrl=$oldUrl;
				$cacheKey='mcf_'.md5($key.$oldUrl);
				$lastCheck=(int)$session->get($cacheKey,0);
				$urlCached=$session->get($cacheKey.'_url',null);
				
				if($checkEvery && $lastCheck && !empty($urlCached) && ($lastCheck+$checkEvery)>time())
				{
					$newUrl=$urlCached;
					$debug[]='OldURl:<code>'.$oldUrl.'</code> will be replaced by NewURL:<code>'.$newUrl.'</code> [Cached...expires:'.date('Y-m-d H:i:s',$lastCheck+$checkEvery).']';
				}
				else
				{
					$urlParams=array();
					//Check if Internal or external
					if($this->params->get('relative',0) && (JString::substr($oldUrl,0,4)=='http' || JString::substr($oldUrl,0,2)=='//'))
					{
						//Remove domain if exists
						if(JString::substr($oldUrl,0,strlen($domain))==$domain)
						{
							$newUrl=str_replace(array($domain,'//'),array('','/'),$newUrl);
							if(JString::substr($newUrl,0,1)!='/') $newUrl='/'.$newUrl;
						}
						else
						{
							//Is extenal source
							$debug[]='OldURl:<code>'.$oldUrl.'</code> is external source & skipped';
							continue;
						}
					}
					
					if($mode==4)
					{
						//Remove other params if exists
						if(JString::strpos($newUrl,'?')!==false)
						{
							list($newUrl,$existingParams)=@explode('?',$newUrl);
						}
					}
					else if($mode==2 || $mode==3)
					{
						if(JString::strpos($newUrl,'jversion=')!==false)
						{
							$debug[]='OldURl:<code>'.$oldUrl.'</code> version parameter already present';
							continue;
						}else{
							//Remove other params if exists
							if(JString::strpos($newUrl,'?')!==false)
							{
								list($newUrl,$existingParams)=@explode('?',$newUrl);
								if(!empty($existingParams) && $mode==2) $urlParams=array_merge(@explode('&',$existingParams),$urlParams);
							}
							$urlParams[]='jversion='.JVERSION;
						}
					}
					else
					{
						//Remove t= param if exists
						if(JString::strpos($newUrl,'t=')!==false)
						{
							$newUrl=preg_replace('/(\?|&)t=([\d]+)/','',$newUrl);
						}
						
						//Remove other params if exists
						if(JString::strpos($newUrl,'?')!==false)
						{
							list($newUrl,$existingParams)=@explode('?',$newUrl);
							if(!empty($existingParams) && $mode==1) $urlParams=array_merge(@explode('&',$existingParams),$urlParams);
						}
						
						//Check if file exists and get Timestamp
						$filePath=JPATH_ROOT.str_replace('/',DIRECTORY_SEPARATOR,$newUrl);
						if(JFile::exists($filePath))
						{
							$urlParams[]='t='.filemtime($filePath);
						}
						else
						{
							$debug[]=$filePath.' not exists';
						}
					}
							
					if(count($urlParams) && $this->params->get('keepqueries',0))
					{
						foreach($urlParams as &$urlParam)
						{
							if(JString::strpos($urlParam,'=')===false) unset($urlParam);
						}
					}
							
					//Apply Parameters
					if(count($urlParams)) $newUrl.='?'.implode('&',$urlParams);
					
					if($checkEvery)
					{
						$session->set($cacheKey.'_url',$newUrl);
						$session->set($cacheKey,$now);
					}
					$debug[]='OldURl:<code>'.$oldUrl.'</code> will be replaced by NewURL:<code>'.$newUrl.'</code>';
				}
								
				//Update Buffer
				if($newUrl!=$oldUrl)
				{
					$buffer=str_replace($oldUrl,$newUrl,$buffer);
				}
			}
		}

		if($allowDebug && count($debug))
		{
			$debug[]=$profiler->mark( ' seconds to complete');
			$buffer=str_replace('</body>','<div style="border:1px dotted red;padding:5px 10px;margin:5px 2px 50px 2px;"><h3>Media Cache Fixer plugin Report</h3><br/>'.implode("<br/>\n",$debug).'</div>'."\n</body>",$buffer);
		}
		JResponse::setBody($buffer);
	}
}